@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 cante100 Loader
 
@@ -49,6 +48,7 @@ import csv
 import os
 import logging
 import xml.etree.ElementTree as ET
+from typing import BinaryIO, cast, Optional, TextIO, Tuple
 
 import librosa
 import numpy as np
@@ -57,6 +57,7 @@ from mirdata import download_utils
 from mirdata import jams_utils
 from mirdata import core
 from mirdata import annotations
+from mirdata import io
 
 
 BIBTEX = """@dataset{nadine_kroher_2018_1322542,
@@ -111,22 +112,20 @@ REMOTES = {
         filename="cante100Meta.xml",
         url="https://zenodo.org/record/1322542/files/cante100Meta.xml?download=1",
         checksum="6cce186ce77a06541cdb9f0a671afb46",  # the md5 checksum
-        destination_dir=None,  # relative path for where to unzip the data, or None
     ),
     "README": download_utils.RemoteFileMetadata(
         filename="cante100_README.txt",
         url="https://zenodo.org/record/1322542/files/cante100_README.txt?download=1",
         checksum="184209b7e7d816fa603f0c7f481c0aae",  # the md5 checksum
-        destination_dir=None,  # relative path for where to unzip the data, or None
     ),
 }
 
 
 DOWNLOAD_INFO = """
         This loader is designed to load the spectrum, as it is available for download.
-        However, the loader supports audio as well. Unfortunately the audio files of the 
-        cante100 dataset are not available for free download, but upon request. However, 
-        you can request de audio in both links here: 
+        However, the loader supports audio as well. Unfortunately the audio files of the
+        cante100 dataset are not available for free download, but upon request. However,
+        you can request de audio in both links here:
         ==> http://www.cofla-project.com/?page_id=208
         ==> https://zenodo.org/record/1324183
         Then, locate the downloaded the cante100audio folder like this:
@@ -144,82 +143,6 @@ were gathered by the COFLA team. COFLA 2015. All rights reserved.
 """
 
 
-def _load_metadata(data_home):
-    metadata_path = os.path.join(data_home, "cante100Meta.xml")
-    if not os.path.exists(metadata_path):
-        logging.info(
-            "Metadata file {} not found.".format(metadata_path)
-            + "You can download the metadata file for cante100 "
-            + "by running cante100.download()"
-        )
-        return None
-
-    tree = ET.parse(metadata_path)
-    root = tree.getroot()
-
-    # ids
-    indexes = []
-    for child in root:
-        index = child.attrib.get("id")
-        if len(index) == 1:
-            index = "00" + index
-            indexes.append(index)
-            continue
-        if len(index) == 2:
-            index = "0" + index
-            indexes.append(index)
-            continue
-        else:
-            indexes.append(index)
-
-    # musicBrainzID
-    identifiers = []
-    for ident in root.iter("musicBrainzID"):
-        identifiers.append(ident.text)
-
-    # artist
-    artists = []
-    for artist in root.iter("artist"):
-        artists.append(artist.text)
-
-    # titles
-    titles = []
-    for title in root.iter("title"):
-        titles.append(title.text)
-
-    # releases
-    releases = []
-    for release in root.iter("anthology"):
-        releases.append(release.text)
-
-    # duration
-    durations = []
-    minutes = []
-    for minute in root.iter("duration_m"):
-        minutes.append(float(minute.text) * 60)
-    seconds = []
-    for second in root.iter("duration_s"):
-        seconds.append(float(second.text))
-    for i in np.arange(len(minutes)):
-        durations.append(minutes[i] + seconds[i])
-
-    metadata = dict()
-    metadata["data_home"] = data_home
-    for i, j in zip(indexes, range(len(artists))):
-        metadata[i] = {
-            "musicBrainzID": identifiers[j],
-            "artist": artists[j],
-            "title": titles[j],
-            "release": releases[j],
-            "duration": durations[j],
-        }
-
-    return metadata
-
-
-DATA = core.LargeData("cante100_index.json", _load_metadata)
-
-
 class Track(core.Track):
     """cante100 track class
 
@@ -242,15 +165,22 @@ class Track(core.Track):
 
     """
 
-    def __init__(self, track_id, data_home):
-        if track_id not in DATA.index["tracks"]:
-            raise ValueError("{} is not a valid track ID in Example".format(track_id))
+    def __init__(
+        self,
+        track_id,
+        data_home,
+        dataset_name,
+        index,
+        metadata,
+    ):
+        super().__init__(
+            track_id,
+            data_home,
+            dataset_name,
+            index,
+            metadata,
+        )
 
-        self.track_id = track_id
-
-        self._data_home = data_home
-
-        self._track_paths = DATA.index["tracks"][track_id]
         self.audio_path = os.path.join(self._data_home, self._track_paths["audio"][0])
         self.spectrogram_path = os.path.join(
             self._data_home, self._track_paths["spectrum"][0]
@@ -258,50 +188,52 @@ class Track(core.Track):
         self.f0_path = os.path.join(self._data_home, self._track_paths["f0"][0])
         self.notes_path = os.path.join(self._data_home, self._track_paths["notes"][0])
 
-        metadata = DATA.metadata(data_home=data_home)
-        if metadata is not None and track_id in metadata:
-            self._track_metadata = metadata[track_id]
-        else:
-            self._track_metadata = {
-                "musicBrainzID": None,
-                "artist": None,
-                "title": None,
-                "release": None,
-                "duration": None,
-            }
-
-        self.identifier = self._track_metadata["musicBrainzID"]
-        self.artist = self._track_metadata["artist"]
-        self.title = self._track_metadata["title"]
-        self.release = self._track_metadata["release"]
-        self.duration = self._track_metadata["duration"]
+    @property
+    def identifier(self):
+        return self._track_metadata.get("musicBrainzID")
 
     @property
-    def audio(self):
+    def artist(self):
+        return self._track_metadata.get("artist")
+
+    @property
+    def title(self):
+        return self._track_metadata.get("title")
+
+    @property
+    def release(self):
+        return self._track_metadata.get("release")
+
+    @property
+    def duration(self):
+        return self._track_metadata.get("duration")
+
+    @property
+    def audio(self) -> Tuple[np.ndarray, float]:
         """The track's audio
 
         Returns:
-           * np.ndarray - audio signal
-           * float - sample rate
+            * np.ndarray - audio signal
+            * float - sample rate
 
         """
         return load_audio(self.audio_path)
 
     @property
-    def spectrogram(self):
+    def spectrogram(self) -> Optional[np.ndarray]:
         """spectrogram of The track's audio
 
         Returns:
-            (np.ndarray): spectrogram
+            np.ndarray: spectrogram
         """
         return load_spectrogram(self.spectrogram_path)
 
     @core.cached_property
-    def melody(self):
+    def melody(self) -> Optional[annotations.F0Data]:
         return load_melody(self.f0_path)
 
     @core.cached_property
-    def notes(self):
+    def notes(self) -> Optional[annotations.NoteData]:
         return load_notes(self.notes_path)
 
     def to_jams(self):
@@ -320,92 +252,82 @@ class Track(core.Track):
         )
 
 
-def load_spectrogram(spectrogram_path):
+@io.coerce_to_string_io
+def load_spectrogram(fhandle: TextIO) -> np.ndarray:
     """Load a cante100 dataset spectrogram file.
 
     Args:
-        spectrogram_path (str): path to audio file
+        fhandle (str or file-like): path or file-like object pointing to an audio file
 
     Returns:
         np.ndarray: spectrogram
 
     """
-    if not os.path.exists(spectrogram_path):
-        raise IOError("spectrogram_path {} does not exist".format(spectrogram_path))
-    parsed_spectrogram = np.genfromtxt(spectrogram_path, delimiter=" ")
-    spectrogram = parsed_spectrogram.astype(np.float)
+    parsed_spectrogram = np.genfromtxt(fhandle, delimiter=" ")
+    spectrogram = parsed_spectrogram.astype(np.float64)
 
     return spectrogram
 
 
-def load_audio(audio_path):
+def load_audio(fhandle: str) -> Tuple[np.ndarray, float]:
     """Load a cante100 audio file.
 
     Args:
-        audio_path (str): path to audio file
+        fhandle (str): path to an audio file
 
     Returns:
         * np.ndarray - the mono audio signal
         * float - The sample rate of the audio file
 
     """
-    if not os.path.exists(audio_path):
-        raise IOError("audio_path {} does not exist".format(audio_path))
-    audio, sr = librosa.load(audio_path, sr=22050, mono=False)
-    return audio, sr
+    return librosa.load(fhandle, sr=22050, mono=False)
 
 
-def load_melody(f0_path):
+@io.coerce_to_string_io
+def load_melody(fhandle: TextIO) -> Optional[annotations.F0Data]:
     """Load cante100 f0 annotations
 
     Args:
-        f0_path (str): path to audio file
+        fhandle (str or file-like): path or file-like object pointing to melody annotation file
 
     Returns:
         F0Data: predominant melody
 
     """
-    if not os.path.exists(f0_path):
-        raise IOError("f0_path {} does not exist".format(f0_path))
-
     times = []
     freqs = []
-    with open(f0_path, "r") as fhandle:
-        reader = csv.reader(fhandle, delimiter=",")
-        for line in reader:
-            times.append(float(line[0]))
-            freqs.append(float(line[1]))
+    reader = csv.reader(fhandle, delimiter=",")
+    for line in reader:
+        times.append(float(line[0]))
+        freqs.append(float(line[1]))
 
-    times = np.array(times)
-    freqs = np.array(freqs)
-    confidence = (freqs > 0).astype(float)
+    times = np.array(times)  # type: ignore
+    freqs = np.array(freqs)  # type: ignore
+    confidence = (cast(np.ndarray, freqs) > 0).astype(float)
 
     return annotations.F0Data(times, freqs, confidence)
 
 
-def load_notes(notes_path):
+@io.coerce_to_string_io
+def load_notes(fhandle: TextIO) -> annotations.NoteData:
     """Load note data from the annotation files
 
     Args:
-        notes_path (str): path to notes file
+        fhandle (str or file-like): path or file-like object pointing to a notes annotation file
 
     Returns:
         NoteData: note annotations
 
     """
-    if not os.path.exists(notes_path):
-        raise IOError("notes_path {} does not exist".format(notes_path))
-
     intervals = []
     pitches = []
     confidence = []
-    with open(notes_path, "r") as fhandle:
-        reader = csv.reader(fhandle, delimiter=",")
-        for line in reader:
-            intervals.append([line[0], float(line[0]) + float(line[1])])
-            # Convert midi value to frequency
-            pitches.append((440 / 32) * (2 ** ((int(line[2]) - 9) / 12)))
-            confidence.append(1.0)
+    reader = csv.reader(fhandle, delimiter=",")
+    for line in reader:
+        intervals.append([line[0], float(line[0]) + float(line[1])])
+        # Convert midi value to frequency
+        pitches.append((440 / 32) * (2 ** ((int(line[2]) - 9) / 12)))
+        confidence.append(1.0)
 
     return annotations.NoteData(
         np.array(intervals, dtype="float"),
@@ -423,15 +345,67 @@ class Dataset(core.Dataset):
     def __init__(self, data_home=None):
         super().__init__(
             data_home,
-            index=DATA.index,
             name="cante100",
-            track_object=Track,
+            track_class=Track,
             bibtex=BIBTEX,
             remotes=REMOTES,
             download_info=DOWNLOAD_INFO,
             license_info=LICENSE_INFO,
         )
 
+    @core.cached_property
+    def _metadata(self):
+        metadata_path = os.path.join(self.data_home, "cante100Meta.xml")
+        if not os.path.exists(metadata_path):
+            raise FileNotFoundError("Metadata not found. Did you run .download()?")
+
+        tree = ET.parse(metadata_path)
+        root = tree.getroot()
+
+        # ids
+        indexes = []
+        for child in root:
+            index = child.attrib.get("id")
+            if len(index) == 1:
+                index = "00" + index
+                indexes.append(index)
+                continue
+            if len(index) == 2:
+                index = "0" + index
+                indexes.append(index)
+                continue
+            else:
+                indexes.append(index)
+
+        # musicBrainzID
+        identifiers = [ident.text for ident in root.iter("musicBrainzID")]
+
+        # artist
+        artists = [artist.text for artist in root.iter("artist")]
+
+        # titles
+        titles = [title.text for title in root.iter("title")]
+
+        # releases
+        releases = [release.text for release in root.iter("anthology")]
+
+        # duration
+        minutes = [float(minute.text) * 60 for minute in root.iter("duration_m")]
+        seconds = [float(second.text) for second in root.iter("duration_s")]
+        durations = [m + s for (m, s) in zip(minutes, seconds)]
+
+        metadata = dict()
+        for i, j in zip(indexes, range(len(artists))):
+            metadata[i] = {
+                "musicBrainzID": identifiers[j],
+                "artist": artists[j],
+                "title": titles[j],
+                "release": releases[j],
+                "duration": durations[j],
+            }
+
+        return metadata
+
     @core.copy_docs(load_audio)
     def load_audio(self, *args, **kwargs):
         return load_audio(*args, **kwargs)
