@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """DALI Dataset Loader
 
 .. admonition:: Dataset Info
@@ -20,6 +19,7 @@ import gzip
 import logging
 import os
 import pickle
+from typing import BinaryIO, Optional, TextIO, Tuple
 
 import librosa
 import numpy as np
@@ -28,6 +28,7 @@ from mirdata import download_utils
 from mirdata import jams_utils
 from mirdata import core
 from mirdata import annotations
+from mirdata import io
 
 # this is the package, needed to load the annotations.
 # DALI-dataset is only installed if the user explicitly declares
@@ -64,9 +65,9 @@ DOWNLOAD_INFO = """
     Once downloaded, unzip the file DALI_v1.0.zip
     and place the result in:
     {}
-    
+
     Use the function dali_code.get_audio you can find at:
-    https://github.com/gabolsgabs/DALI for getting the audio 
+    https://github.com/gabolsgabs/DALI for getting the audio
     and place them in "audio" folder with the following structure:
     > Dali
         > audio
@@ -78,21 +79,6 @@ LICENSE_INFO = (
 )
 
 
-def _load_metadata(data_home):
-    metadata_path = os.path.join(data_home, os.path.join("dali_metadata.json"))
-    if not os.path.exists(metadata_path):
-        logging.info("Metadata file {} not found.".format(metadata_path))
-        return None
-    with open(metadata_path, "r") as fhandle:
-        metadata_index = json.load(fhandle)
-
-    metadata_index["data_home"] = data_home
-    return metadata_index
-
-
-DATA = core.LargeData("dali_index.json", _load_metadata)
-
-
 class Track(core.Track):
     """DALI melody Track class
 
@@ -124,82 +110,102 @@ class Track(core.Track):
 
     """
 
-    def __init__(self, track_id, data_home):
-        if track_id not in DATA.index["tracks"]:
-            raise ValueError("{} is not a valid track ID in DALI".format(track_id))
+    def __init__(
+        self,
+        track_id,
+        data_home,
+        dataset_name,
+        index,
+        metadata,
+    ):
+        super().__init__(
+            track_id,
+            data_home,
+            dataset_name,
+            index,
+            metadata,
+        )
 
-        self.track_id = track_id
-        self._data_home = data_home
-        self._track_paths = DATA.index["tracks"][track_id]
         self.annotation_path = os.path.join(
             self._data_home, self._track_paths["annot"][0]
         )
+        self.audio_path = os.path.join(self._data_home, self._track_paths["audio"][0])
 
-        metadata = DATA.metadata(data_home)
-        if metadata is not None and track_id in metadata:
-            self._track_metadata = metadata[track_id]
-            self._track_metadata["album"] = metadata[track_id]["metadata"]["album"]
-            self._track_metadata["release_date"] = metadata[track_id]["metadata"][
-                "release_date"
-            ]
-            self._track_metadata["language"] = metadata[track_id]["metadata"][
-                "language"
-            ]
-            self.audio_url = self._track_metadata["audio"]["url"]
-            self.url_working = self._track_metadata["audio"]["working"]
-            self.ground_truth = self._track_metadata["ground-truth"]
-            self.artist = self._track_metadata["artist"]
-            self.title = self._track_metadata["title"]
-            self.dataset_version = self._track_metadata["dataset_version"]
-            self.scores_ncc = self._track_metadata["scores"]["NCC"]
-            self.scores_manual = self._track_metadata["scores"]["manual"]
-            self.album = self._track_metadata["album"]
-            self.release_date = self._track_metadata["release_date"]
-            self.language = self._track_metadata["language"]
-            self.audio_path = os.path.join(
-                self._data_home, self._track_paths["audio"][0]
-            )
-        else:
-            self.audio_url = None
-            self.url_working = None
-            self.ground_truth = None
-            self.artist = None
-            self.title = None
-            self.dataset_version = None
-            self.scores_ncc = None
-            self.scores_manual = None
-            self.album = None
-            self.release_date = None
-            self.language = None
-            self.audio_path = None
+    @property
+    def audio_url(self):
+        return self._track_metadata.get("audio", {}).get("url")
+
+    @property
+    def url_working(self):
+        return self._track_metadata.get("audio", {}).get("working")
+
+    @property
+    def ground_truth(self):
+        return self._track_metadata.get("ground-truth")
+
+    @property
+    def artist(self):
+        return self._track_metadata.get("artist")
+
+    @property
+    def title(self):
+        return self._track_metadata.get("title")
+
+    @property
+    def dataset_version(self):
+        return self._track_metadata.get("dataset_version")
+
+    @property
+    def scores_ncc(self):
+        return self._track_metadata.get("scores", {}).get("NCC")
+
+    @property
+    def scores_manual(self):
+        return self._track_metadata.get("scores", {}).get("manual")
+
+    @property
+    def album(self):
+        return self._track_metadata.get("metadata", {}).get("album")
+
+    @property
+    def release_date(self):
+        return self._track_metadata.get("metadata", {}).get("release_date")
+
+    @property
+    def genres(self):
+        return self._track_metadata.get("metadata", {}).get("genres")
+
+    @property
+    def language(self):
+        return self._track_metadata.get("metadata", {}).get("language")
 
     @core.cached_property
-    def notes(self):
+    def notes(self) -> annotations.NoteData:
         return load_annotations_granularity(self.annotation_path, "notes")
 
     @core.cached_property
-    def words(self):
+    def words(self) -> annotations.NoteData:
         return load_annotations_granularity(self.annotation_path, "words")
 
     @core.cached_property
-    def lines(self):
+    def lines(self) -> annotations.NoteData:
         return load_annotations_granularity(self.annotation_path, "lines")
 
     @core.cached_property
-    def paragraphs(self):
+    def paragraphs(self) -> annotations.NoteData:
         return load_annotations_granularity(self.annotation_path, "paragraphs")
 
     @core.cached_property
-    def annotation_object(self):
+    def annotation_object(self) -> DALI.Annotations:
         return load_annotations_class(self.annotation_path)
 
     @property
-    def audio(self):
+    def audio(self) -> Optional[Tuple[np.ndarray, float]]:
         """The track's audio
 
         Returns:
-           * np.ndarray - audio signal
-           * float - sample rate
+            * np.ndarray - audio signal
+            * float - sample rate
 
         """
         return load_audio(self.audio_path)
@@ -223,20 +229,19 @@ class Track(core.Track):
         )
 
 
-def load_audio(audio_path):
+@io.coerce_to_bytes_io
+def load_audio(fhandle: BinaryIO) -> Optional[Tuple[np.ndarray, float]]:
     """Load a DALI audio file.
 
     Args:
-        audio_path (str): path to audio file
+        fhandle (str or file-like): path or file-like object pointing to an audio file
 
     Returns:
         * np.ndarray - the mono audio signal
         * float - The sample rate of the audio file
 
     """
-    if not os.path.exists(audio_path):
-        raise IOError("audio_path {} does not exist".format(audio_path))
-    return librosa.load(audio_path, sr=None, mono=True)
+    return librosa.load(fhandle, sr=None, mono=True)
 
 
 def load_annotations_granularity(annotations_path, granularity):
@@ -250,9 +255,6 @@ def load_annotations_granularity(annotations_path, granularity):
         NoteData for granularity='notes' or LyricData otherwise
 
     """
-    if not os.path.exists(annotations_path):
-        raise IOError("annotations_path {} does not exist".format(annotations_path))
-
     try:
         with gzip.open(annotations_path, "rb") as f:
             output = pickle.load(f)
@@ -309,15 +311,25 @@ class Dataset(core.Dataset):
     def __init__(self, data_home=None):
         super().__init__(
             data_home,
-            index=DATA.index,
             name="dali",
-            track_object=Track,
+            track_class=Track,
             bibtex=BIBTEX,
             remotes=REMOTES,
             download_info=DOWNLOAD_INFO,
             license_info=LICENSE_INFO,
         )
 
+    @core.cached_property
+    def _metadata(self):
+        metadata_path = os.path.join(self.data_home, os.path.join("dali_metadata.json"))
+        if not os.path.exists(metadata_path):
+            raise FileNotFoundError("Metadata not found. Did you run .download()?")
+
+        with open(metadata_path, "r") as fhandle:
+            metadata_index = json.load(fhandle)
+
+        return metadata_index
+
     @core.copy_docs(load_audio)
     def load_audio(self, *args, **kwargs):
         return load_audio(*args, **kwargs)
