@@ -1,13 +1,12 @@
-# -*- coding: utf-8 -*-
 """Medley-solos-DB Dataset Loader.
 
 .. admonition:: Dataset Info
     :class: dropdown
 
     Medley-solos-DB is a cross-collection dataset for automatic musical instrument
-    recognition in solo recordings. It consists of a training set of 3-second audio 
-    clips, which are extracted from the MedleyDB dataset (Bittner et al., ISMIR 2014) 
-    as well as a test set of 3-second clips, which are extracted from the solosDB 
+    recognition in solo recordings. It consists of a training set of 3-second audio
+    clips, which are extracted from the MedleyDB dataset (Bittner et al., ISMIR 2014)
+    as well as a test set of 3-second clips, which are extracted from the solosDB
     dataset (Essid et al., IEEE TASLP 2009).
 
     Each of these clips contains a single instrument among a taxonomy of eight:
@@ -28,13 +27,17 @@
 """
 
 import csv
-import librosa
 import logging
 import os
+from typing import BinaryIO, Optional, TextIO, Tuple
+
+import librosa
+import numpy as np
 
 from mirdata import download_utils
 from mirdata import jams_utils
 from mirdata import core
+from mirdata import io
 
 BIBTEX = """@inproceedings{lostanlen2019ismir,
     title={Deep Convolutional Networks in the Pitch Spiral for Musical Instrument Recognition},
@@ -60,36 +63,6 @@ REMOTES = {
 LICENSE_INFO = "Creative Commons Attribution 4.0 International."
 
 
-def _load_metadata(data_home):
-    metadata_path = os.path.join(
-        data_home, "annotation", "Medley-solos-DB_metadata.csv"
-    )
-
-    if not os.path.exists(metadata_path):
-        logging.info("Metadata file {} not found.".format(metadata_path))
-        return None
-
-    metadata_index = {}
-    with open(metadata_path, "r") as fhandle:
-        csv_reader = csv.reader(fhandle, delimiter=",")
-        next(csv_reader)
-        for row in csv_reader:
-            subset, instrument_str, instrument_id, song_id, track_id = row
-            metadata_index[str(track_id)] = {
-                "subset": str(subset),
-                "instrument": str(instrument_str),
-                "instrument_id": int(instrument_id),
-                "song_id": int(song_id),
-            }
-
-    metadata_index["data_home"] = data_home
-
-    return metadata_index
-
-
-DATA = core.LargeData("medley_solos_db_index.json", _load_metadata)
-
-
 class Track(core.Track):
     """medley_solos_db Track class
 
@@ -106,42 +79,47 @@ class Track(core.Track):
 
     """
 
-    def __init__(self, track_id, data_home):
-        if track_id not in DATA.index["tracks"]:
-            raise ValueError(
-                "{} is not a valid track ID in Medley-solos-DB".format(track_id)
-            )
-
-        self.track_id = track_id
-
-        self._data_home = data_home
-        self._track_paths = DATA.index["tracks"][track_id]
-
-        metadata = DATA.metadata(data_home)
-        if metadata is not None and track_id in metadata:
-            self._track_metadata = metadata[track_id]
-        else:
-            self._track_metadata = {
-                "instrument": None,
-                "instrument_id": None,
-                "song_id": None,
-                "subset": None,
-                "track_id": None,
-            }
+    def __init__(
+        self,
+        track_id,
+        data_home,
+        dataset_name,
+        index,
+        metadata,
+    ):
+        super().__init__(
+            track_id,
+            data_home,
+            dataset_name,
+            index,
+            metadata,
+        )
 
         self.audio_path = os.path.join(self._data_home, self._track_paths["audio"][0])
-        self.instrument = self._track_metadata["instrument"]
-        self.instrument_id = self._track_metadata["instrument_id"]
-        self.song_id = self._track_metadata["song_id"]
-        self.subset = self._track_metadata["subset"]
 
     @property
-    def audio(self):
+    def instrument(self):
+        return self._track_metadata.get("instrument")
+
+    @property
+    def instrument_id(self):
+        return self._track_metadata.get("instrument_id")
+
+    @property
+    def song_id(self):
+        return self._track_metadata.get("song_id")
+
+    @property
+    def subset(self):
+        return self._track_metadata.get("subset")
+
+    @property
+    def audio(self) -> Optional[Tuple[np.ndarray, float]]:
         """The track's audio
 
         Returns:
-           * np.ndarray - audio signal
-           * float - sample rate
+            * np.ndarray - audio signal
+            * float - sample rate
 
         """
         return load_audio(self.audio_path)
@@ -158,21 +136,19 @@ class Track(core.Track):
         )
 
 
-def load_audio(audio_path):
+@io.coerce_to_bytes_io
+def load_audio(fhandle: BinaryIO) -> Tuple[np.ndarray, float]:
     """Load a Medley Solos DB audio file.
 
     Args:
-        audio_path (str): path to audio file
+        fhandle (str or file-like): File-like object or path to audio file
 
     Returns:
         * np.ndarray - the mono audio signal
         * float - The sample rate of the audio file
 
     """
-    if not os.path.exists(audio_path):
-        raise IOError("audio_path {} does not exist".format(audio_path))
-
-    return librosa.load(audio_path, sr=22050, mono=True)
+    return librosa.load(fhandle, sr=22050, mono=True)
 
 
 @core.docstring_inherit(core.Dataset)
@@ -184,14 +160,37 @@ class Dataset(core.Dataset):
     def __init__(self, data_home=None):
         super().__init__(
             data_home,
-            index=DATA.index,
             name="medley_solos_db",
-            track_object=Track,
+            track_class=Track,
             bibtex=BIBTEX,
             remotes=REMOTES,
             license_info=LICENSE_INFO,
         )
 
+    @core.cached_property
+    def _metadata(self):
+        metadata_path = os.path.join(
+            self.data_home, "annotation", "Medley-solos-DB_metadata.csv"
+        )
+
+        if not os.path.exists(metadata_path):
+            raise FileNotFoundError("Metadata not found. Did you run .download()?")
+
+        metadata_index = {}
+        with open(metadata_path, "r") as fhandle:
+            csv_reader = csv.reader(fhandle, delimiter=",")
+            next(csv_reader)
+            for row in csv_reader:
+                subset, instrument_str, instrument_id, song_id, track_id = row
+                metadata_index[str(track_id)] = {
+                    "subset": str(subset),
+                    "instrument": str(instrument_str),
+                    "instrument_id": int(instrument_id),
+                    "song_id": int(song_id),
+                }
+
+        return metadata_index
+
     @core.copy_docs(load_audio)
     def load_audio(self, *args, **kwargs):
         return load_audio(*args, **kwargs)
