@@ -1,11 +1,10 @@
-# -*- coding: utf-8 -*-
 """MedleyDB pitch Dataset Loader
 
 .. admonition:: Dataset Info
     :class: dropdown
 
-    MedleyDB Pitch is a pitch-tracking subset of the MedleyDB dataset 
-    containing only f0-annotated, monophonic stems. 
+    MedleyDB Pitch is a pitch-tracking subset of the MedleyDB dataset
+    containing only f0-annotated, monophonic stems.
 
     MedleyDB is a dataset of annotated, royalty-free multitrack recordings.
     MedleyDB was curated primarily to support research on melody extraction,
@@ -19,15 +18,18 @@
 
 import csv
 import json
-import librosa
 import logging
-import numpy as np
 import os
+from typing import BinaryIO, cast, Optional, TextIO, Tuple
+
+import librosa
+import numpy as np
 
 from mirdata import download_utils
 from mirdata import jams_utils
 from mirdata import core
 from mirdata import annotations
+from mirdata import io
 
 
 BIBTEX = """@inproceedings{bittner2014medleydb,
@@ -41,7 +43,7 @@ DOWNLOAD_INFO = """
     To download this dataset, visit:
     https://zenodo.org/record/2620624#.XKZc7hNKh24
     and request access.
-    
+
     Once downloaded, unzip the file MedleyDB-Pitch.zip
     and copy the result to:
     {}
@@ -52,23 +54,6 @@ LICENSE_INFO = (
 )
 
 
-def _load_metadata(data_home):
-    metadata_path = os.path.join(data_home, "medleydb_pitch_metadata.json")
-
-    if not os.path.exists(metadata_path):
-        logging.info("Metadata file {} not found.".format(metadata_path))
-        return None
-
-    with open(metadata_path, "r") as fhandle:
-        metadata = json.load(fhandle)
-
-    metadata["data_home"] = data_home
-    return metadata
-
-
-DATA = core.LargeData("medleydb_pitch_index.json", _load_metadata)
-
-
 class Track(core.Track):
     """medleydb_pitch Track class
 
@@ -89,46 +74,53 @@ class Track(core.Track):
 
     """
 
-    def __init__(self, track_id, data_home):
-        if track_id not in DATA.index["tracks"]:
-            raise ValueError(
-                "{} is not a valid track ID in MedleyDB-Pitch".format(track_id)
-            )
+    def __init__(
+        self,
+        track_id,
+        data_home,
+        dataset_name,
+        index,
+        metadata,
+    ):
+        super().__init__(
+            track_id,
+            data_home,
+            dataset_name,
+            index,
+            metadata,
+        )
 
-        self.track_id = track_id
-
-        self._data_home = data_home
-        self._track_paths = DATA.index["tracks"][track_id]
         self.pitch_path = os.path.join(self._data_home, self._track_paths["pitch"][0])
 
-        metadata = DATA.metadata(data_home)
-        if metadata is not None and track_id in metadata:
-            self._track_metadata = metadata[track_id]
-        else:
-            self._track_metadata = {
-                "instrument": None,
-                "artist": None,
-                "title": None,
-                "genre": None,
-            }
-
         self.audio_path = os.path.join(self._data_home, self._track_paths["audio"][0])
-        self.instrument = self._track_metadata["instrument"]
-        self.artist = self._track_metadata["artist"]
-        self.title = self._track_metadata["title"]
-        self.genre = self._track_metadata["genre"]
+
+    @property
+    def instrument(self):
+        return self._track_metadata.get("instrument")
+
+    @property
+    def artist(self):
+        return self._track_metadata.get("artist")
+
+    @property
+    def title(self):
+        return self._track_metadata.get("title")
+
+    @property
+    def genre(self):
+        return self._track_metadata.get("genre")
 
     @core.cached_property
-    def pitch(self):
+    def pitch(self) -> Optional[annotations.F0Data]:
         return load_pitch(self.pitch_path)
 
     @property
-    def audio(self):
+    def audio(self) -> Optional[Tuple[np.ndarray, float]]:
         """The track's audio
 
         Returns:
-           * np.ndarray - audio signal
-           * float - sample rate
+            * np.ndarray - audio signal
+            * float - sample rate
 
         """
         return load_audio(self.audio_path)
@@ -147,24 +139,23 @@ class Track(core.Track):
         )
 
 
-def load_audio(audio_path):
+@io.coerce_to_bytes_io
+def load_audio(fhandle: BinaryIO) -> Tuple[np.ndarray, float]:
     """Load a MedleyDB audio file.
 
     Args:
-        audio_path (str): path to audio file
+        fhandle (str or file-like): File-like object or path to audio file
 
     Returns:
         * np.ndarray - the mono audio signal
         * float - The sample rate of the audio file
 
     """
-    if not os.path.exists(audio_path):
-        raise IOError("audio_path {} does not exist".format(audio_path))
-
-    return librosa.load(audio_path, sr=None, mono=True)
+    return librosa.load(fhandle, sr=None, mono=True)
 
 
-def load_pitch(pitch_path):
+@io.coerce_to_string_io
+def load_pitch(fhandle: TextIO) -> annotations.F0Data:
     """load a MedleyDB pitch annotation file
 
     Args:
@@ -177,20 +168,17 @@ def load_pitch(pitch_path):
         F0Data: pitch annotation
 
     """
-    if not os.path.exists(pitch_path):
-        raise IOError("pitch_path {} does not exist".format(pitch_path))
 
     times = []
     freqs = []
-    with open(pitch_path, "r") as fhandle:
-        reader = csv.reader(fhandle, delimiter=",")
-        for line in reader:
-            times.append(float(line[0]))
-            freqs.append(float(line[1]))
+    reader = csv.reader(fhandle, delimiter=",")
+    for line in reader:
+        times.append(float(line[0]))
+        freqs.append(float(line[1]))
 
-    times = np.array(times)
-    freqs = np.array(freqs)
-    confidence = (freqs > 0).astype(float)
+    times = np.array(times)  # type: ignore
+    freqs = np.array(freqs)  # type: ignore
+    confidence = (cast(np.ndarray, freqs) > 0).astype(float)
     pitch_data = annotations.F0Data(times, freqs, confidence)
     return pitch_data
 
@@ -204,14 +192,25 @@ class Dataset(core.Dataset):
     def __init__(self, data_home=None):
         super().__init__(
             data_home,
-            index=DATA.index,
             name="medleydb_pitch",
-            track_object=Track,
+            track_class=Track,
             bibtex=BIBTEX,
             download_info=DOWNLOAD_INFO,
             license_info=LICENSE_INFO,
         )
 
+    @core.cached_property
+    def _metadata(self):
+        metadata_path = os.path.join(self.data_home, "medleydb_pitch_metadata.json")
+
+        if not os.path.exists(metadata_path):
+            raise FileNotFoundError("Metadata not found. Did you run .download()?")
+
+        with open(metadata_path, "r") as fhandle:
+            metadata = json.load(fhandle)
+
+        return metadata
+
     @core.copy_docs(load_audio)
     def load_audio(self, *args, **kwargs):
         return load_audio(*args, **kwargs)
