@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """RWC Classical Dataset Loader
 
 .. admonition:: Dataset Info
@@ -37,7 +36,7 @@
         96: 3rd beat
 
     In 6/8 time signature, they correspond as follows:
-    
+
     .. code-block:: latex
 
         384: 1st beat in a measure (i.e., downbeat position)
@@ -53,6 +52,7 @@
 import csv
 import logging
 import os
+from typing import BinaryIO, Optional, TextIO, Tuple
 
 import librosa
 import numpy as np
@@ -61,6 +61,7 @@ from mirdata import download_utils
 from mirdata import jams_utils
 from mirdata import core
 from mirdata import annotations
+from mirdata import io
 
 BIBTEX = """@inproceedings{goto2002rwc,
   title={RWC Music Database: Popular, Classical and Jazz Music Databases.},
@@ -86,7 +87,6 @@ REMOTES = {
         filename="master.zip",
         url="https://github.com/magdalenafuentes/metadata/archive/master.zip",
         checksum="7dbe87fedbaaa1f348625a2af1d78030",
-        destination_dir="",
     ),
 }
 DOWNLOAD_INFO = """
@@ -103,62 +103,16 @@ DOWNLOAD_INFO = """
 LICENSE_INFO = """
 From the dataset's owner webpage:
 
-'Users who have submitted the Pledge and received authorization may freely use the database for research purposes 
-without facing the usual copyright restrictions, but all of the copyrights and neighboring rights connected with 
-this database belong to the National Institute of Advanced Industrial Science and Technology and are managed by the 
-RWC Music Database Administrator. Persons or organizations that have not submitted a Pledge and that have not 
+'Users who have submitted the Pledge and received authorization may freely use the database for research purposes
+without facing the usual copyright restrictions, but all of the copyrights and neighboring rights connected with
+this database belong to the National Institute of Advanced Industrial Science and Technology and are managed by the
+RWC Music Database Administrator. Persons or organizations that have not submitted a Pledge and that have not
 received authorization may not use the database.'
 
 See https://staff.aist.go.jp/m.goto/RWC-MDB/ for more details.
 """
 
 
-def _load_metadata(data_home):
-
-    metadata_path = os.path.join(data_home, "metadata-master", "rwc-c.csv")
-
-    if not os.path.exists(metadata_path):
-        logging.info(
-            "Metadata file {} not found.".format(metadata_path)
-            + "You can download the metadata file by running download()"
-        )
-        return None
-
-    with open(metadata_path, "r") as fhandle:
-        dialect = csv.Sniffer().sniff(fhandle.read(1024))
-        fhandle.seek(0)
-        reader = csv.reader(fhandle, dialect)
-        raw_data = []
-        for line in reader:
-            if line[0] != "Piece No.":
-                raw_data.append(line)
-
-    metadata_index = {}
-    for line in raw_data:
-        if line[0] == "Piece No.":
-            continue
-        p = "00" + line[0].split(".")[1][1:]
-        track_id = "RM-C{}".format(p[len(p) - 3 :])
-
-        metadata_index[track_id] = {
-            "piece_number": line[0],
-            "suffix": line[1],
-            "track_number": line[2],
-            "title": line[3],
-            "composer": line[4],
-            "artist": line[5],
-            "duration": _duration_to_sec(line[6]),
-            "category": line[7],
-        }
-
-    metadata_index["data_home"] = data_home
-
-    return metadata_index
-
-
-DATA = core.LargeData("rwc_classical_index.json", _load_metadata)
-
-
 class Track(core.Track):
     """rwc_classical Track class
 
@@ -186,61 +140,76 @@ class Track(core.Track):
 
     """
 
-    def __init__(self, track_id, data_home):
-        if track_id not in DATA.index["tracks"]:
-            raise ValueError(
-                "{} is not a valid track ID in rwc_classical".format(track_id)
-            )
+    def __init__(
+        self,
+        track_id,
+        data_home,
+        dataset_name,
+        index,
+        metadata,
+    ):
+        super().__init__(
+            track_id,
+            data_home,
+            dataset_name,
+            index,
+            metadata,
+        )
 
-        self.track_id = track_id
-        self._data_home = data_home
-        self._track_paths = DATA.index["tracks"][track_id]
         self.sections_path = os.path.join(
             self._data_home, self._track_paths["sections"][0]
         )
         self.beats_path = os.path.join(self._data_home, self._track_paths["beats"][0])
 
-        metadata = DATA.metadata(data_home)
-        if metadata is not None and track_id in metadata:
-            self._track_metadata = metadata[track_id]
-        else:
-            self._track_metadata = {
-                "piece_number": None,
-                "suffix": None,
-                "track_number": None,
-                "title": None,
-                "composer": None,
-                "artist": None,
-                "duration": None,
-                "category": None,
-            }
-
         self.audio_path = os.path.join(self._data_home, self._track_paths["audio"][0])
 
-        self.piece_number = self._track_metadata["piece_number"]
-        self.suffix = self._track_metadata["suffix"]
-        self.track_number = self._track_metadata["track_number"]
-        self.title = self._track_metadata["title"]
-        self.composer = self._track_metadata["composer"]
-        self.artist = self._track_metadata["artist"]
-        self.duration = self._track_metadata["duration"]
-        self.category = self._track_metadata["category"]
+    @property
+    def piece_number(self):
+        return self._track_metadata.get("piece_number")
+
+    @property
+    def suffix(self):
+        return self._track_metadata.get("suffix")
+
+    @property
+    def track_number(self):
+        return self._track_metadata.get("track_number")
+
+    @property
+    def title(self):
+        return self._track_metadata.get("title")
+
+    @property
+    def composer(self):
+        return self._track_metadata.get("composer")
+
+    @property
+    def artist(self):
+        return self._track_metadata.get("artist")
+
+    @property
+    def duration(self):
+        return self._track_metadata.get("duration")
+
+    @property
+    def category(self):
+        return self._track_metadata.get("category")
 
     @core.cached_property
-    def sections(self):
+    def sections(self) -> Optional[annotations.SectionData]:
         return load_sections(self.sections_path)
 
     @core.cached_property
-    def beats(self):
+    def beats(self) -> Optional[annotations.BeatData]:
         return load_beats(self.beats_path)
 
     @property
-    def audio(self):
+    def audio(self) -> Optional[Tuple[np.ndarray, float]]:
         """The track's audio
 
         Returns:
-           * np.ndarray - audio signal
-           * float - sample rate
+            * np.ndarray - audio signal
+            * float - sample rate
 
         """
         return load_audio(self.audio_path)
@@ -260,46 +229,41 @@ class Track(core.Track):
         )
 
 
-def load_audio(audio_path):
+@io.coerce_to_bytes_io
+def load_audio(fhandle: BinaryIO) -> Tuple[np.ndarray, float]:
     """Load a RWC audio file.
 
     Args:
-        audio_path (str): path to audio file
+        fhandle (str or file-like): File-like object or path to audio file
 
     Returns:
         * np.ndarray - the mono audio signal
         * float - The sample rate of the audio file
 
     """
-    if not os.path.exists(audio_path):
-        raise IOError("audio_path {} does not exist".format(audio_path))
-
-    return librosa.load(audio_path, sr=None, mono=True)
+    return librosa.load(fhandle, sr=None, mono=True)
 
 
-def load_sections(sections_path):
+@io.coerce_to_string_io
+def load_sections(fhandle: TextIO) -> Optional[annotations.SectionData]:
     """Load rwc section data from a file
 
     Args:
-        sections_path (str): path to sections annotation file
+        fhandle (str or file-like): File-like object or path to sections annotation file
 
     Returns:
         SectionData: section data
 
     """
-    if not os.path.exists(sections_path):
-        raise IOError("sections_path {} does not exist".format(sections_path))
-
     begs = []  # timestamps of section beginnings
     ends = []  # timestamps of section endings
     secs = []  # section labels
 
-    with open(sections_path, "r") as fhandle:
-        reader = csv.reader(fhandle, delimiter="\t")
-        for line in reader:
-            begs.append(float(line[0]) / 100.0)
-            ends.append(float(line[1]) / 100.0)
-            secs.append(line[2])
+    reader = csv.reader(fhandle, delimiter="\t")
+    for line in reader:
+        begs.append(float(line[0]) / 100.0)
+        ends.append(float(line[1]) / 100.0)
+        secs.append(line[2])
 
     if not begs:  # some files are empty
         return None
@@ -315,8 +279,8 @@ def _position_in_bar(beat_positions, beat_times):
         beat_times (np.ndarray): raw rwc time stamps
 
     Returns:
-        * np.ndarray: normalized beat positions
-        * np.ndarray: normalized time stamps
+        * np.ndarray - normalized beat positions
+        * np.ndarray - normalized time stamps
 
     """
     # Remove -1
@@ -344,32 +308,30 @@ def _position_in_bar(beat_positions, beat_times):
     return beat_positions_corrected, beat_times_corrected
 
 
-def load_beats(beats_path):
+@io.coerce_to_string_io
+def load_beats(fhandle: TextIO) -> annotations.BeatData:
     """Load rwc beat data from a file
 
     Args:
-        beats_path (str): path to beats annotation file
+        fhandle (str or file-like): File-like object or path to beats annotation file
 
     Returns:
         BeatData: beat data
 
     """
-    if not os.path.exists(beats_path):
-        raise IOError("beats_path {} does not exist".format(beats_path))
 
     beat_times = []  # timestamps of beat interval beginnings
     beat_positions = []  # beat position inside the bar
 
-    with open(beats_path, "r") as fhandle:
-        reader = csv.reader(fhandle, delimiter="\t")
-        for line in reader:
-            beat_times.append(float(line[0]) / 100.0)
-            beat_positions.append(int(line[2]))
-    beat_positions, beat_times = _position_in_bar(
+    reader = csv.reader(fhandle, delimiter="\t")
+    for line in reader:
+        beat_times.append(float(line[0]) / 100.0)
+        beat_positions.append(int(line[2]))
+    beat_positions_in_bar, beat_times = _position_in_bar(
         np.array(beat_positions), np.array(beat_times)
     )
 
-    return annotations.BeatData(beat_times, beat_positions.astype(int))
+    return annotations.BeatData(beat_times, beat_positions_in_bar.astype(int))
 
 
 def _duration_to_sec(duration):
@@ -407,15 +369,51 @@ class Dataset(core.Dataset):
     def __init__(self, data_home=None):
         super().__init__(
             data_home,
-            index=DATA.index,
             name="rwc_classical",
-            track_object=Track,
+            track_class=Track,
             bibtex=BIBTEX,
             remotes=REMOTES,
             download_info=DOWNLOAD_INFO,
             license_info=LICENSE_INFO,
         )
 
+    @core.cached_property
+    def _metadata(self):
+
+        metadata_path = os.path.join(self.data_home, "metadata-master", "rwc-c.csv")
+
+        if not os.path.exists(metadata_path):
+            raise FileNotFoundError("Metadata not found. Did you run .download()?")
+
+        with open(metadata_path, "r") as fhandle:
+            dialect = csv.Sniffer().sniff(fhandle.read(1024))
+            fhandle.seek(0)
+            reader = csv.reader(fhandle, dialect)
+            raw_data = []
+            for line in reader:
+                if line[0] != "Piece No.":
+                    raw_data.append(line)
+
+        metadata_index = {}
+        for line in raw_data:
+            if line[0] == "Piece No.":
+                continue
+            p = "00" + line[0].split(".")[1][1:]
+            track_id = "RM-C{}".format(p[len(p) - 3 :])
+
+            metadata_index[track_id] = {
+                "piece_number": line[0],
+                "suffix": line[1],
+                "track_number": line[2],
+                "title": line[3],
+                "composer": line[4],
+                "artist": line[5],
+                "duration": _duration_to_sec(line[6]),
+                "category": line[7],
+            }
+
+        return metadata_index
+
     @core.copy_docs(load_audio)
     def load_audio(self, *args, **kwargs):
         return load_audio(*args, **kwargs)
