@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 import importlib
 import inspect
 from inspect import signature
@@ -12,7 +10,7 @@ import requests
 
 import mirdata
 from mirdata import core, download_utils
-from tests.test_utils import DEFAULT_DATA_HOME
+from tests.test_utils import DEFAULT_DATA_HOME, get_attributes_and_properties
 
 DATASETS = mirdata.DATASETS
 CUSTOM_TEST_TRACKS = {
@@ -21,6 +19,7 @@ CUSTOM_TEST_TRACKS = {
     "giantsteps_key": "3",
     "dali": "4b196e6c99574dd49ad00d56e132712b",
     "giantsteps_tempo": "113",
+    "gtzan_genre": "country.00000",
     "guitarset": "03_BN3-119-G_solo",
     "irmas": "1",
     "medley_solos_db": "d07b1fc0-567d-52c2-fef4-239f31c9d40e",
@@ -43,42 +42,13 @@ REMOTE_DATASETS = {
         "remote_checksum": "c5fbdd4f8b7de383796a34143cb44c4f",
     }
 }
+TEST_DATA_HOME = "tests/resources/mir_datasets"
 
 
-def create_remote_index(httpserver, dataset_name):
-    httpserver.serve_content(
-        open(REMOTE_DATASETS[dataset_name]["local_index"], "rb").read()
-    )
-    remote_index = {
-        "index": download_utils.RemoteFileMetadata(
-            filename=REMOTE_DATASETS[dataset_name]["remote_filename"],
-            url=httpserver.url,
-            checksum=REMOTE_DATASETS[dataset_name]["remote_checksum"],
-            destination_dir="",
-        )
-    }
-    data_remote = core.LargeData(
-        REMOTE_DATASETS[dataset_name]["filename"], remote_index=remote_index
-    )
-    return data_remote.index
-
-
-def clean_remote_dataset(dataset_name):
-    os.remove(
-        os.path.join(
-            "mirdata/datasets/indexes", REMOTE_DATASETS[dataset_name]["filename"]
-        )
-    )
-
-
-def test_dataset_attributes(httpserver):
+def test_dataset_attributes():
     for dataset_name in DATASETS:
         module = importlib.import_module("mirdata.datasets.{}".format(dataset_name))
-        if dataset_name not in REMOTE_DATASETS:
-            dataset = module.Dataset()
-        else:
-            remote_index = create_remote_index(httpserver, dataset_name)
-            dataset = module.Dataset(index=remote_index)
+        dataset = module.Dataset(os.path.join(TEST_DATA_HOME, dataset_name))
 
         assert (
             dataset.name == dataset_name
@@ -87,7 +57,7 @@ def test_dataset_attributes(httpserver):
             dataset.bibtex is not None
         ), "No BIBTEX information provided for {}".format(dataset_name)
         assert (
-                dataset._license_info is not None
+            dataset._license_info is not None
         ), "No LICENSE information provided for {}".format(dataset_name)
         assert (
             isinstance(dataset.remotes, dict) or dataset.remotes is None
@@ -98,25 +68,18 @@ def test_dataset_attributes(httpserver):
         assert (
             isinstance(dataset._download_info, str) or dataset._download_info is None
         ), "{}.DOWNLOAD_INFO must be a string".format(dataset_name)
-        assert type(dataset._track_object) == type(
+        assert type(dataset._track_class) == type(
             core.Track
         ), "{}.Track must be an instance of core.Track".format(dataset_name)
         assert callable(dataset.download), "{}.download is not a function".format(
             dataset_name
         )
 
-        if dataset_name in REMOTE_DATASETS:
-            clean_remote_dataset(dataset_name)
 
-
-def test_cite_and_license(httpserver):
+def test_cite_and_license():
     for dataset_name in DATASETS:
         module = importlib.import_module("mirdata.datasets.{}".format(dataset_name))
-        if dataset_name not in REMOTE_DATASETS:
-            dataset = module.Dataset()
-        else:
-            remote_index = create_remote_index(httpserver, dataset_name)
-            dataset = module.Dataset(index=remote_index)
+        dataset = module.Dataset(os.path.join(TEST_DATA_HOME, dataset_name))
 
         text_trap = io.StringIO()
         sys.stdout = text_trap
@@ -127,23 +90,17 @@ def test_cite_and_license(httpserver):
         sys.stdout = text_trap
         dataset.license()
         sys.stdout = sys.__stdout__
-        if dataset_name in REMOTE_DATASETS:
-            clean_remote_dataset(dataset_name)
 
 
 KNOWN_ISSUES = {}  # key is module, value is REMOTE key
-DOWNLOAD_EXCEPTIONS = ["maestro", "acousticbrainz_genre"]
+DOWNLOAD_EXCEPTIONS = ["maestro"]
 
 
-def test_download(mocker, httpserver):
+def test_download(mocker):
     for dataset_name in DATASETS:
         print(dataset_name)
         module = importlib.import_module("mirdata.datasets.{}".format(dataset_name))
-        if dataset_name not in REMOTE_DATASETS:
-            dataset = module.Dataset()
-        else:
-            remote_index = create_remote_index(httpserver, dataset_name)
-            dataset = module.Dataset(index=remote_index)
+        dataset = module.Dataset(os.path.join(TEST_DATA_HOME, dataset_name))
 
         # test parameters & defaults
         assert callable(dataset.download), "{}.download is not callable".format(
@@ -200,24 +157,16 @@ def test_download(mocker, httpserver):
                 dataset.download()
             except:
                 assert False, "{}: {}".format(dataset_name, sys.exc_info()[0])
-        if dataset_name in REMOTE_DATASETS:
-            clean_remote_dataset(dataset_name)
 
 
 # This is magically skipped by the the remote fixture `skip_local` in conftest.py
 # when tests are run with the --local flag
-def test_validate(skip_local, httpserver):
+def test_validate(skip_local):
     for dataset_name in DATASETS:
         data_home = os.path.join("tests/resources/mir_datasets", dataset_name)
 
         module = importlib.import_module("mirdata.datasets.{}".format(dataset_name))
-        if dataset_name not in REMOTE_DATASETS:
-            dataset = module.Dataset(data_home)
-            dataset_default = module.Dataset(data_home=None)
-        else:
-            remote_index = create_remote_index(httpserver, dataset_name)
-            dataset = module.Dataset(data_home, index=remote_index)
-            dataset_default = module.Dataset(data_home=None, index=remote_index)
+        dataset = module.Dataset(os.path.join(TEST_DATA_HOME, dataset_name))
 
         try:
             dataset.validate()
@@ -229,25 +178,12 @@ def test_validate(skip_local, httpserver):
         except:
             assert False, "{}: {}".format(dataset_name, sys.exc_info()[0])
 
-        try:
-            dataset_default.validate(verbose=False)
-        except:
-            assert False, "{}: {}".format(dataset_name, sys.exc_info()[0])
 
-
-def test_load_and_trackids(httpserver):
+def test_load_and_trackids():
     for dataset_name in DATASETS:
         data_home = os.path.join("tests/resources/mir_datasets", dataset_name)
         module = importlib.import_module("mirdata.datasets.{}".format(dataset_name))
-        if dataset_name not in REMOTE_DATASETS:
-            dataset = module.Dataset(data_home)
-            dataset_default = module.Dataset()
-        else:
-            continue
-            # TODO - fix the dataset object to work with remote index
-            # remote_index = create_remote_index(httpserver, dataset_name)
-            # dataset = module.Dataset(data_home, index=remote_index)
-            # dataset_default = module.Dataset(index=remote_index)
+        dataset = module.Dataset(os.path.join(TEST_DATA_HOME, dataset_name))
 
         try:
             track_ids = dataset.track_ids
@@ -258,7 +194,7 @@ def test_load_and_trackids(httpserver):
         )
         trackid_len = len(track_ids)
         # if the dataset has tracks, test the loaders
-        if dataset._track_object is not None:
+        if dataset._track_class is not None:
 
             try:
                 choice_track = dataset.choice_track()
@@ -275,8 +211,8 @@ def test_load_and_trackids(httpserver):
             except:
                 assert False, "{}: {}".format(dataset_name, sys.exc_info()[0])
 
-            assert (
-                type(dataset_data) is dict
+            assert isinstance(
+                dataset_data, dict
             ), "{}.load should return a dictionary".format(dataset_name)
             assert (
                 len(dataset_data.keys()) == trackid_len
@@ -284,43 +220,19 @@ def test_load_and_trackids(httpserver):
                 dataset_name, dataset_name
             )
 
-            try:
-                dataset_data_default = dataset_default.load_tracks()
-            except:
-                assert False, "{}: {}".format(dataset_name, sys.exc_info()[0])
 
-            assert (
-                type(dataset_data_default) is dict
-            ), "{}.load should return a dictionary".format(dataset_name)
-            assert (
-                len(dataset_data_default.keys()) == trackid_len
-            ), "the dictionary returned {}.load() does not have the same number of elements as {}.track_ids()".format(
-                dataset_name, dataset_name
-            )
-        if dataset_name in REMOTE_DATASETS:
-            clean_remote_dataset(dataset_name)
-
-
-def test_track(httpserver):
+def test_track():
     data_home_dir = "tests/resources/mir_datasets"
 
     for dataset_name in DATASETS:
         data_home = os.path.join(data_home_dir, dataset_name)
 
         module = importlib.import_module("mirdata.datasets.{}".format(dataset_name))
-        if dataset_name not in REMOTE_DATASETS:
-            dataset = module.Dataset(data_home)
-            dataset_default = module.Dataset()
-        else:
-            continue
-            # TODO - fix the dataset object to work with remote index
-            # remote_index = create_remote_index(httpserver, dataset_name)
-            # dataset = module.Dataset(data_home, index=remote_index)
-            # dataset_default = module.Dataset(index=remote_index)
+        dataset = module.Dataset(os.path.join(TEST_DATA_HOME, dataset_name))
 
         # if the dataset doesn't have a track object, make sure it raises a value error
         # and move on to the next dataset
-        if dataset._track_object is None:
+        if dataset._track_class is None:
             with pytest.raises(NotImplementedError):
                 dataset.track("~faketrackid~?!")
             continue
@@ -330,15 +242,6 @@ def test_track(httpserver):
         else:
             trackid = dataset.track_ids[0]
 
-        try:
-            track_default = dataset_default.track(trackid)
-        except:
-            assert False, "{}: {}".format(dataset_name, sys.exc_info()[0])
-
-        assert track_default._data_home == os.path.join(
-            DEFAULT_DATA_HOME, dataset.name
-        ), "{}: Track._data_home path is not set as expected".format(dataset_name)
-
         # test data home specified
         try:
             track_test = dataset.track(trackid)
@@ -353,6 +256,18 @@ def test_track(httpserver):
             track_test, "to_jams"
         ), "{}.track must have a to_jams method".format(dataset_name)
 
+        # test calling all attributes, properties and cached properties
+        track_data = get_attributes_and_properties(track_test)
+
+        for attr in track_data["attributes"]:
+            ret = getattr(track_test, attr)
+
+        for prop in track_data["properties"]:
+            ret = getattr(track_test, prop)
+
+        for cprop in track_data["cached_properties"]:
+            ret = getattr(track_test, cprop)
+
         # Validate JSON schema
         try:
             jam = track_test.to_jams()
@@ -376,6 +291,46 @@ def test_track(httpserver):
             dataset.track("~faketrackid~?!")
 
 
+# This tests the case where there is no data in data_home.
+# It makes sure that the track can be initialized and the
+# attributes accessed, but that anything requiring data
+# files errors (all properties and cached properties).
+def test_track_placeholder_case():
+    data_home_dir = "not/a/real/path"
+
+    for dataset_name in DATASETS:
+        data_home = os.path.join(data_home_dir, dataset_name)
+
+        module = importlib.import_module("mirdata.datasets.{}".format(dataset_name))
+        dataset = module.Dataset(os.path.join(data_home, dataset_name))
+
+        if dataset._track_class is None or dataset.remote_index:
+            continue
+
+        if dataset_name in CUSTOM_TEST_TRACKS:
+            trackid = CUSTOM_TEST_TRACKS[dataset_name]
+        else:
+            trackid = dataset.track_ids[0]
+
+        try:
+            track_test = dataset.track(trackid)
+        except:
+            assert False, "{}: {}".format(dataset_name, sys.exc_info()[0])
+
+        track_data = get_attributes_and_properties(track_test)
+
+        for attr in track_data["attributes"]:
+            ret = getattr(track_test, attr)
+
+        for prop in track_data["properties"]:
+            with pytest.raises(Exception):
+                ret = getattr(track_test, prop)
+
+        for cprop in track_data["cached_properties"]:
+            with pytest.raises(Exception):
+                ret = getattr(track_test, cprop)
+
+
 # for load_* functions which require more than one argument
 # module_name : {function_name: {parameter2: value, parameter3: value}}
 EXCEPTIONS = {
@@ -405,14 +360,10 @@ SKIP = {
 }
 
 
-def test_load_methods(httpserver):
+def test_load_methods():
     for dataset_name in DATASETS:
         module = importlib.import_module("mirdata.datasets.{}".format(dataset_name))
-        if dataset_name not in REMOTE_DATASETS:
-            dataset = module.Dataset()
-        else:
-            remote_index = create_remote_index(httpserver, dataset_name)
-            dataset = module.Dataset(index=remote_index)
+        dataset = module.Dataset(os.path.join(TEST_DATA_HOME, dataset_name))
 
         all_methods = dir(dataset)
         load_methods = [
@@ -430,7 +381,11 @@ def test_load_methods(httpserver):
                 continue
 
             if load_method.__doc__ is None:
-                raise ValueError("{} has no documentation".format(method_name))
+                raise ValueError(
+                    "mirdata.datasets.{}.Dataset.{} has no documentation".format(
+                        dataset_name, method_name
+                    )
+                )
 
             params = [
                 p
@@ -452,17 +407,13 @@ def test_load_methods(httpserver):
 CUSTOM_TEST_MTRACKS = {}
 
 
-def test_multitracks(httpserver):
+def test_multitracks():
     data_home_dir = "tests/resources/mir_datasets"
 
     for dataset_name in DATASETS:
 
         module = importlib.import_module("mirdata.datasets.{}".format(dataset_name))
-        if dataset_name not in REMOTE_DATASETS:
-            dataset = module.Dataset()
-        else:
-            remote_index = create_remote_index(httpserver, dataset_name)
-            dataset = module.Dataset(index=remote_index)
+        dataset = module.Dataset(os.path.join(TEST_DATA_HOME, dataset_name))
 
         # TODO this is currently an opt-in test. Make it an opt out test
         # once #265 is addressed
@@ -504,5 +455,3 @@ def test_multitracks(httpserver):
         assert jam.validate(), "Jams validation failed for {}.MultiTrack({})".format(
             dataset_name, mtrack_id
         )
-        if dataset_name in REMOTE_DATASETS:
-            clean_remote_dataset(dataset_name)
